@@ -25,15 +25,19 @@
 KItemListRoleEditor::KItemListRoleEditor(QWidget *parent) :
     KTextEdit(parent),
     m_index(0),
-    m_role()
+    m_role(),
+    m_blockFinishedSignal(false)
 {
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    setAcceptRichText(false);
     document()->setDocumentMargin(0);
 
     if (parent) {
         parent->installEventFilter(this);
     }
+
+    connect(this, SIGNAL(textChanged()), this, SLOT(autoAdjustSize()));
 }
 
 KItemListRoleEditor::~KItemListRoleEditor()
@@ -60,10 +64,10 @@ QByteArray KItemListRoleEditor::role() const
     return m_role;
 }
 
-bool  KItemListRoleEditor::eventFilter(QObject* watched, QEvent* event)
+bool KItemListRoleEditor::eventFilter(QObject* watched, QEvent* event)
 {
     if (watched == parentWidget() && event->type() == QEvent::Resize) {
-        emit roleEditingFinished(m_index, m_role, toPlainText());
+        emitRoleEditingFinished();
     }
 
     return KTextEdit::eventFilter(watched, event);
@@ -72,7 +76,10 @@ bool  KItemListRoleEditor::eventFilter(QObject* watched, QEvent* event)
 bool KItemListRoleEditor::event(QEvent* event)
 {
     if (event->type() == QEvent::FocusOut) {
-        emit roleEditingFinished(m_index, m_role, toPlainText());
+        QFocusEvent* focusEvent = static_cast<QFocusEvent*>(event);
+        if (focusEvent->reason() != Qt::PopupFocusReason) {
+            emitRoleEditingFinished();
+        }
     }
     return KTextEdit::event(event);
 }
@@ -81,12 +88,18 @@ void KItemListRoleEditor::keyPressEvent(QKeyEvent* event)
 {
     switch (event->key()) {
     case Qt::Key_Escape:
+        // Emitting the signal roleEditingCanceled might result
+        // in losing the focus. Per default losing the focus emits
+        // a roleEditingFinished signal (see KItemListRoleEditor::event),
+        // which is not wanted in this case.
+        m_blockFinishedSignal = true;
         emit roleEditingCanceled(m_index, m_role, toPlainText());
+        m_blockFinishedSignal = false;
         event->accept();
         return;
     case Qt::Key_Enter:
     case Qt::Key_Return:
-        emit roleEditingFinished(m_index, m_role, toPlainText());
+        emitRoleEditingFinished();
         event->accept();
         return;
     default:
@@ -94,7 +107,6 @@ void KItemListRoleEditor::keyPressEvent(QKeyEvent* event)
     }
 
     KTextEdit::keyPressEvent(event);
-    autoAdjustSize();
 }
 
 void KItemListRoleEditor::autoAdjustSize()
@@ -122,4 +134,11 @@ void KItemListRoleEditor::autoAdjustSize()
     }
 }
 
+void KItemListRoleEditor::emitRoleEditingFinished()
+{
+    if (!m_blockFinishedSignal) {
+        emit roleEditingFinished(m_index, m_role, toPlainText());
+    }
+}
+
 #include "kitemlistroleeditor.moc"
