@@ -62,6 +62,16 @@ PlacesPanel::~PlacesPanel()
 
 bool PlacesPanel::urlChanged()
 {
+    if (!url().isValid() || url().protocol().contains("search")) {
+        // Skip results shown by a search, as possible identical
+        // directory names are useless without parent-path information.
+        return false;
+    }
+
+    if (m_controller) {
+        selectClosestItem();
+    }
+
     return true;
 }
 
