@@ -1,5 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2008 by David Faure <faure@kde.org>                     *
+ *   Copyright (C) 2012 by Peter Penz <peter.penz19@gmail.com>             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -19,7 +20,8 @@
 
 #include "dolphinviewactionhandler.h"
 
-#include "additionalinfoaccessor.h"
+#include <config-nepomuk.h>
+
 #include "settings/viewpropertiesdialog.h"
 #include "views/dolphinview.h"
 #include "views/zoomlevelinfo.h"
@@ -28,20 +30,27 @@
 #include <KAction>
 #include <KActionCollection>
 #include <KActionMenu>
-#include <KFileItemDelegate>
+#include <kitemviews/kfileitemmodel.h>
 #include <KLocale>
+#include <KMenu>
 #include <KNewFileMenu>
 #include <KSelectAction>
 #include <KToggleAction>
-#include <KRun>
 #include <KPropertiesDialog>
+#include <KIcon>
+
+#ifdef HAVE_NEPOMUK
+    #include <Nepomuk/ResourceManager>
+#endif
 
 #include <KDebug>
 
-DolphinViewActionHandler::DolphinViewActionHandler(KActionCollection* collection, QObject* parent)
-    : QObject(parent),
-      m_actionCollection(collection),
-      m_currentView(0)
+DolphinViewActionHandler::DolphinViewActionHandler(KActionCollection* collection, QObject* parent) :
+    QObject(parent),
+    m_actionCollection(collection),
+    m_currentView(0),
+    m_sortByActions(),
+    m_visibleRoles()
 {
     Q_ASSERT(m_actionCollection);
     createActions();
@@ -65,16 +74,14 @@ void DolphinViewActionHandler::setCurrentView(DolphinView* view)
             this, SLOT(slotSortOrderChanged(Qt::SortOrder)));
     connect(view, SIGNAL(sortFoldersFirstChanged(bool)),
             this, SLOT(slotSortFoldersFirstChanged(bool)));
-    connect(view, SIGNAL(additionalInfoListChanged(QList<DolphinView::AdditionalInfo>,
-                                                   QList<DolphinView::AdditionalInfo>)),
-            this, SLOT(slotAdditionalInfoListChanged(QList<DolphinView::AdditionalInfo>,
-                                                     QList<DolphinView::AdditionalInfo>)));
+    connect(view, SIGNAL(visibleRolesChanged(QList<QByteArray>,QList<QByteArray>)),
+            this, SLOT(slotVisibleRolesChanged(QList<QByteArray>,QList<QByteArray>)));
     connect(view, SIGNAL(groupedSortingChanged(bool)),
             this, SLOT(slotGroupedSortingChanged(bool)));
     connect(view, SIGNAL(hiddenFilesShownChanged(bool)),
             this, SLOT(slotHiddenFilesShownChanged(bool)));
-    connect(view, SIGNAL(sortingChanged(DolphinView::Sorting)),
-            this, SLOT(slotSortingChanged(DolphinView::Sorting)));
+    connect(view, SIGNAL(sortRoleChanged(QByteArray)),
+            this, SLOT(slotSortRoleChanged(QByteArray)));
     connect(view, SIGNAL(zoomLevelChanged(int,int)),
             this, SLOT(slotZoomLevelChanged(int,int)));
 }
@@ -169,8 +176,7 @@ void DolphinViewActionHandler::createActions()
     connect(sortFoldersFirst, SIGNAL(triggered()), this, SLOT(toggleSortFoldersFirst()));
 
     // View -> Sort By
-    QActionGroup* sortByActionGroup = createSortByActionGroup();
-    connect(sortByActionGroup, SIGNAL(triggered(QAction*)), this, SLOT(slotSortTriggered(QAction*)));
+    QActionGroup* sortByActionGroup = createFileItemRolesActionGroup("sort_by_");
 
     KActionMenu* sortByActionMenu = m_actionCollection->add<KActionMenu>("sort");
     sortByActionMenu->setText(i18nc("@action:inmenu View", "Sort By"));
@@ -184,14 +190,14 @@ void DolphinViewActionHandler::createActions()
     sortByActionMenu->addAction(sortFoldersFirst);
 
     // View -> Additional Information
-    QActionGroup* additionalInfoGroup = createAdditionalInformationActionGroup();
-    connect(additionalInfoGroup, SIGNAL(triggered(QAction*)), this, SLOT(toggleAdditionalInfo(QAction*)));
+    QActionGroup* visibleRolesGroup = createFileItemRolesActionGroup("show_");
 
-    KActionMenu* additionalInfoMenu = m_actionCollection->add<KActionMenu>("additional_info");
-    additionalInfoMenu->setText(i18nc("@action:inmenu View", "Additional Information"));
-    additionalInfoMenu->setDelayed(false);
-    foreach (QAction* action, additionalInfoGroup->actions()) {
-        additionalInfoMenu->addAction(action);
+    KActionMenu* visibleRolesMenu = m_actionCollection->add<KActionMenu>("additional_info");
+    visibleRolesMenu->setText(i18nc("@action:inmenu View", "Additional Information"));
+    visibleRolesMenu->setDelayed(false);
+
+    foreach (QAction* action, visibleRolesGroup->actions()) {
+        visibleRolesMenu->addAction(action);
     }
 
     KToggleAction* showInGroups = m_actionCollection->add<KToggleAction>("show_in_groups");
@@ -209,53 +215,85 @@ void DolphinViewActionHandler::createActions()
     connect(adjustViewProps, SIGNAL(triggered()), this, SLOT(slotAdjustViewProperties()));
 }
 
-QActionGroup* DolphinViewActionHandler::createAdditionalInformationActionGroup()
+QActionGroup* DolphinViewActionHandler::createFileItemRolesActionGroup(const QString& groupPrefix)
 {
-    QActionGroup* additionalInfoGroup = new QActionGroup(m_actionCollection);
-    additionalInfoGroup->setExclusive(false);
+    const bool isSortGroup = (groupPrefix == QLatin1String("sort_by_"));
+    Q_ASSERT(isSortGroup || (!isSortGroup && groupPrefix == QLatin1String("show_")));
 
-    KActionMenu* showInformationMenu = m_actionCollection->add<KActionMenu>("additional_info");
-    showInformationMenu->setText(i18nc("@action:inmenu View", "Additional Information"));
-    showInformationMenu->setDelayed(false);
-
-    const AdditionalInfoAccessor& infoAccessor = AdditionalInfoAccessor::instance();
-
-    const QList<DolphinView::AdditionalInfo> infoList = infoAccessor.keys();
-    foreach (DolphinView::AdditionalInfo info, infoList) {
-        const QString name = infoAccessor.actionCollectionName(info, AdditionalInfoAccessor::AdditionalInfoType);
-        KToggleAction* action = m_actionCollection->add<KToggleAction>(name);
-        action->setText(infoAccessor.translation(info));
-        action->setData(info);
-        action->setActionGroup(additionalInfoGroup);
+    QActionGroup* rolesActionGroup = new QActionGroup(m_actionCollection);
+    rolesActionGroup->setExclusive(isSortGroup);
+    if (isSortGroup) {
+        connect(rolesActionGroup, SIGNAL(triggered(QAction*)),
+                this, SLOT(slotSortTriggered(QAction*)));
+    } else {
+        connect(rolesActionGroup, SIGNAL(triggered(QAction*)),
+                this, SLOT(toggleVisibleRole(QAction*)));
     }
 
-    return additionalInfoGroup;
-}
+    QString groupName;
+    KActionMenu* groupMenu = 0;
+    QActionGroup* groupMenuGroup = 0;
 
-Q_DECLARE_METATYPE(DolphinView::Sorting)
+    bool nepomukRunning = false;
+    bool indexingEnabled = false;
+#ifdef HAVE_NEPOMUK
+    nepomukRunning = (Nepomuk::ResourceManager::instance()->initialized());
+    if (nepomukRunning) {
+        KConfig config("nepomukserverrc");
+        indexingEnabled = config.group("Service-nepomukfileindexer").readEntry("autostart", false);
+    }
+#endif
 
-QActionGroup* DolphinViewActionHandler::createSortByActionGroup()
-{
-    QActionGroup* sortByActionGroup = new QActionGroup(m_actionCollection);
-    sortByActionGroup->setExclusive(true);
+    const QList<KFileItemModel::RoleInfo> rolesInfo = KFileItemModel::rolesInformation();
+    foreach (const KFileItemModel::RoleInfo& info, rolesInfo) {
+        if (!isSortGroup && info.role == "text") {
+            // It should not be possible to hide the "text" role
+            continue;
+        }
 
-    KToggleAction* sortByName = m_actionCollection->add<KToggleAction>("sort_by_name");
-    sortByName->setText(i18nc("@action:inmenu Sort By", "Name"));
-    sortByName->setData(QVariant::fromValue(DolphinView::SortByName));
-    sortByActionGroup->addAction(sortByName);
+        KToggleAction* action = 0;
+        const QString name = groupPrefix + info.role;
+        if (info.group.isEmpty()) {
+            action = m_actionCollection->add<KToggleAction>(name);
+            action->setActionGroup(rolesActionGroup);
+        } else {
+            if (!groupMenu || info.group != groupName) {
+                groupName = info.group;
+                groupMenu = m_actionCollection->add<KActionMenu>(groupName);
+                groupMenu->setText(groupName);
+                groupMenu->setActionGroup(rolesActionGroup);
 
-    const AdditionalInfoAccessor& infoAccessor = AdditionalInfoAccessor::instance();
-    const QList<DolphinView::AdditionalInfo> infoList = infoAccessor.keys();
-    foreach (DolphinView::AdditionalInfo info, infoList) {
-        const QString name = infoAccessor.actionCollectionName(info, AdditionalInfoAccessor::SortByType);
-        KToggleAction* action = m_actionCollection->add<KToggleAction>(name);
-        action->setText(infoAccessor.translation(info));
-        const DolphinView::Sorting sorting = infoAccessor.sorting(info);
-        action->setData(QVariant::fromValue(sorting));
-        sortByActionGroup->addAction(action);
+                groupMenuGroup = new QActionGroup(groupMenu);
+                groupMenuGroup->setExclusive(isSortGroup);
+                if (isSortGroup) {
+                    connect(groupMenuGroup, SIGNAL(triggered(QAction*)),
+                            this, SLOT(slotSortTriggered(QAction*)));
+                } else {
+                    connect(groupMenuGroup, SIGNAL(triggered(QAction*)),
+                            this, SLOT(toggleVisibleRole(QAction*)));
+                }
+            }
+
+            action = new KToggleAction(groupMenu);
+            action->setActionGroup(groupMenuGroup);
+            groupMenu->addAction(action);
+        }
+        action->setText(info.translation);
+        action->setData(info.role);
+
+        const bool enable = (!info.requiresNepomuk && !info.requiresIndexer) ||
+                            (info.requiresNepomuk && nepomukRunning) ||
+                            (info.requiresIndexer && indexingEnabled);
+        action->setEnabled(enable);
+
+        if (isSortGroup) {
+            m_sortByActions.insert(info.role, action);
+        } else {
+            m_visibleRoles.insert(info.role, action);
+        }
     }
 
-    return sortByActionGroup;
+    return rolesActionGroup;
 }
 
 void DolphinViewActionHandler::slotViewModeActionTriggered(QAction* action)
@@ -342,9 +380,9 @@ void DolphinViewActionHandler::updateViewActions()
 
     slotSortOrderChanged(m_currentView->sortOrder());
     slotSortFoldersFirstChanged(m_currentView->sortFoldersFirst());
-    slotAdditionalInfoListChanged(m_currentView->additionalInfoList(), QList<DolphinView::AdditionalInfo>());
+    slotVisibleRolesChanged(m_currentView->visibleRoles(), QList<QByteArray>());
     slotGroupedSortingChanged(m_currentView->groupedSorting());
-    slotSortingChanged(m_currentView->sorting());
+    slotSortRoleChanged(m_currentView->sortRole());
     slotZoomLevelChanged(m_currentView->zoomLevel(), -1);
 
     QAction* showHiddenFilesAction = m_actionCollection->action("show_hidden_files");
@@ -391,44 +429,40 @@ void DolphinViewActionHandler::slotSortFoldersFirstChanged(bool foldersFirst)
     m_actionCollection->action("folders_first")->setChecked(foldersFirst);
 }
 
-void DolphinViewActionHandler::toggleAdditionalInfo(QAction* action)
+void DolphinViewActionHandler::toggleVisibleRole(QAction* action)
 {
     emit actionBeingHandled();
 
-    const DolphinView::AdditionalInfo info =
-        static_cast<DolphinView::AdditionalInfo>(action->data().toInt());
+    const QByteArray toggledRole = action->data().toByteArray();
 
-    QList<DolphinView::AdditionalInfo> list = m_currentView->additionalInfoList();
+    QList<QByteArray> roles = m_currentView->visibleRoles();
 
     const bool show = action->isChecked();
 
-    const int index = list.indexOf(info);
+    const int index = roles.indexOf(toggledRole);
     const bool containsInfo = (index >= 0);
     if (show && !containsInfo) {
-        list.append(info);
-        m_currentView->setAdditionalInfoList(list);
+        roles.append(toggledRole);
+        m_currentView->setVisibleRoles(roles);
     } else if (!show && containsInfo) {
-        list.removeAt(index);
-        m_currentView->setAdditionalInfoList(list);
-        Q_ASSERT(list.indexOf(info) < 0);
+        roles.removeAt(index);
+        m_currentView->setVisibleRoles(roles);
+        Q_ASSERT(roles.indexOf(toggledRole) < 0);
     }
 }
 
-void DolphinViewActionHandler::slotAdditionalInfoListChanged(const QList<DolphinView::AdditionalInfo>& current,
-                                                             const QList<DolphinView::AdditionalInfo>& previous)
+void DolphinViewActionHandler::slotVisibleRolesChanged(const QList<QByteArray>& current,
+                                                       const QList<QByteArray>& previous)
 {
     Q_UNUSED(previous);
 
-    const AdditionalInfoAccessor& infoAccessor = AdditionalInfoAccessor::instance();
-
-    const QList<DolphinView::AdditionalInfo> checkedInfo = current;
-    const QList<DolphinView::AdditionalInfo> infoList = infoAccessor.keys();
-
-    foreach (DolphinView::AdditionalInfo info, infoList) {
-        const QString name = infoAccessor.actionCollectionName(info, AdditionalInfoAccessor::AdditionalInfoType);
-        QAction* action = m_actionCollection->action(name);
-        Q_ASSERT(action);
-        action->setChecked(checkedInfo.contains(info));
+    const QSet<QByteArray> checkedRoles = current.toSet();
+    QHashIterator<QByteArray, KToggleAction*> it(m_visibleRoles);
+    while (it.hasNext()) {
+        it.next();
+        const QByteArray& role = it.key();
+        KToggleAction* action = it.value();
+        action->setChecked(checkedRoles.contains(role));
     }
 }
 
@@ -488,23 +522,9 @@ KToggleAction* DolphinViewActionHandler::detailsModeAction()
     return detailsView;
 }
 
-void DolphinViewActionHandler::slotSortingChanged(DolphinView::Sorting sorting)
+void DolphinViewActionHandler::slotSortRoleChanged(const QByteArray& role)
 {
-    QAction* action = 0;
-    if (sorting == DolphinView::SortByName) {
-        action = m_actionCollection->action("sort_by_name");
-    } else {
-        const AdditionalInfoAccessor& infoAccessor = AdditionalInfoAccessor::instance();
-        const QList<DolphinView::AdditionalInfo> infoList = infoAccessor.keys();
-        foreach (DolphinView::AdditionalInfo info, infoList) {
-            if (sorting == infoAccessor.sorting(info)) {
-                const QString name = infoAccessor.actionCollectionName(info, AdditionalInfoAccessor::SortByType);
-                action = m_actionCollection->action(name);
-                break;
-            }
-        }
-    }
-
+    KToggleAction* action = m_sortByActions.value(role);
     if (action) {
         action->setChecked(true);
 
@@ -530,8 +550,27 @@ void DolphinViewActionHandler::slotZoomLevelChanged(int current, int previous)
 
 void DolphinViewActionHandler::slotSortTriggered(QAction* action)
 {
-    const DolphinView::Sorting sorting = action->data().value<DolphinView::Sorting>();
-    m_currentView->setSorting(sorting);
+    // The radiobuttons of the "Sort By"-menu are split between the main-menu
+    // and several sub-menus. Because of this they don't have a common
+    // action-group that assures an exclusive toggle-state between the main-menu
+    // actions and the sub-menu-actions. If an action gets checked, it must
+    // be assured that all other actions get unchecked.
+    QAction* sortByMenu =  m_actionCollection->action("sort");
+    foreach (QAction* groupAction, sortByMenu->menu()->actions()) {
+        KActionMenu* actionMenu = qobject_cast<KActionMenu*>(groupAction);
+        if (actionMenu) {
+            foreach (QAction* subAction, actionMenu->menu()->actions()) {
+                subAction->setChecked(false);
+            }
+        } else if (groupAction->actionGroup()) {
+            groupAction->setChecked(false);
+        }
+    }
+    action->setChecked(true);
+
+    // Apply the activated sort-role to the view
+    const QByteArray role = action->data().toByteArray();
+    m_currentView->setSortRole(role);
 }
 
 void DolphinViewActionHandler::slotAdjustViewProperties()
