@@ -219,7 +219,11 @@ void KonqView::switchView( KonqViewFactory &viewFactory )
 {
     //kDebug();
   KParts::ReadOnlyPart *oldPart = m_pPart;
-  m_pPart = m_pKonqFrame->attach( viewFactory ); // creates the part
+  KParts::ReadOnlyPart *part = m_pKonqFrame->attach( viewFactory ); // creates the part
+  if (!part) {
+      return;
+  }
+  m_pPart = part;
 
   // Set the statusbar in the BE asap to avoid a KMainWindow statusbar being created.
   KParts::StatusBarExtension* sbext = statusBarExtension();
@@ -514,6 +518,15 @@ void KonqView::slotRequestFocus( KParts::ReadOnlyPart * )
 void KonqView::setLoading( bool loading, bool hasPending /*= false*/)
 {
     //kDebug() << "loading=" << loading << "hasPending=" << hasPending;
+    if (loading) {
+        // Make sure the focus is restored on the part's widget and not the combo
+        // box if it starts loading a request. See #304933.
+        QWidget* partWidget = (m_pPart ? m_pPart->widget() : 0);
+        if (partWidget && !partWidget->hasFocus()) {
+            //kDebug() << "SET FOCUS on the widget";
+            partWidget->setFocus();
+        }
+    }
     m_bLoading = loading;
     m_bPendingRedirection = hasPending;
     if ( m_pMainWindow->currentView() == this )
